import pytest
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


def test_claude_info(runner):
    """Check info spots running process uv"""
    result = runner.invoke(cli, ["claude", "info"])
    assert result.exit_code == 0
    assert "uv" in result.stdout



import os
import shutil
from pathlib import Path

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


@pytest.fixture(autouse=True)
def backup_env():
    """Fixture to automatically backup and restore .env file."""
    env_path = Path(__file__).parent.parent / ".env"
    bak_path = Path(__file__).parent.parent / ".env.bak"
    if env_path.is_file():
        shutil.copy2(env_path, bak_path)
    yield
    if bak_path.is_file():
        shutil.move(bak_path, env_path)


def test_dot_env(runner):
    """Check missing entry in .env leads to validation error."""

    result = runner.invoke(cli, ["health"])
    assert result.exit_code == 0

    # Read .env, remove STARBRIDGE_ATLASSIAN_URL line and write back
    env_path = Path(__file__).parent.parent / ".env"
    with open(env_path) as f:
        lines = f.readlines()

    with open(env_path, "w") as f:
        for line in lines:
            if not line.startswith("STARBRIDGE_ATLASSIAN_URL"):
                f.write(line)
    os.environ.pop("STARBRIDGE_ATLASSIAN_URL", None)

    result = runner.invoke(cli, ["health"])
    assert result.exit_code == 78
    assert "STARBRIDGE_ATLASSIAN_URL: Field required" in result.output



from .console import console
from .health import AggregatedHealth, Health
from .logging import get_logger
from .settings import load_settings
from .signature import description_and_params

__all__ = [
    "console",
    "get_logger",
    "description_and_params",
    "load_settings",
    "Health",
    "AggregatedHealth",
]



import json
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge.base import __version__
from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


def test_built_with_love(runner):
    """Check epilog shown."""
    result = runner.invoke(cli, ["--help"])
    assert result.exit_code == 0
    assert "built with love in Berlin" in result.output


def test_invalid_command(runner):
    """Test invalid command returns error"""
    result = runner.invoke(cli, ["invalid"])
    assert result.exit_code != 0


def test_info(runner):
    """Check processes exposed and version matching."""
    result = runner.invoke(cli, ["info"])
    assert result.exit_code == 0
    assert "'uv'" in result.stdout
    assert f"'version': '{__version__}'" in result.stdout


def test_install(runner, tmp_path):
    """Check processes exposed and version matching."""
    with patch(
        "starbridge.claude.service.Service.application_directory", return_value=tmp_path
    ):
        inputs = (
            "https://your-domain.atlassian.net/test\n"
            "you-test@your-domain.com\n"  # Atlassian email address
            "YOUR_TEST_API_TOKEN\n"  # Atlassian token
            "\n"  # Possibly other prompt input
        )
        result = runner.invoke(cli, ["install", "--no-restart-claude"], input=inputs)
        assert result.exit_code == 0

        result = runner.invoke(cli, ["claude", "config"], input=inputs)
        assert result.exit_code == 0
        # Find start of JSON by looking for first '{'
        json_start = result.output.find("{")
        output_json = json.loads(result.output[json_start:])
        server_config = output_json["mcpServers"]["starbridge"]
        assert (
            server_config["env"]["STARBRIDGE_ATLASSIAN_URL"]
            == "https://your-domain.atlassian.net/test"
        )
        assert (
            server_config["env"]["STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS"]
            == "you-test@your-domain.com"
        )
        assert (
            server_config["env"]["STARBRIDGE_ATLASSIAN_API_TOKEN"]
            == "YOUR_TEST_API_TOKEN"
        )

        result = runner.invoke(cli, ["uninstall", "--no-restart-claude"], input=inputs)
        assert result.exit_code == 0

        result = runner.invoke(cli, ["claude", "config"], input=inputs)
        assert result.exit_code == 0
        json_start = result.output.find("{")
        output_json = json.loads(result.output[json_start:])
        assert output_json["mcpServers"].get("starbridge") is None



import pytest
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


def test_mcp_services(runner):
    """Check available services."""
    result = runner.invoke(cli, ["mcp", "services"])
    assert result.exit_code == 0

    # We expect these three services to be present
    expected_services = [
        "starbridge.claude.service.Service",
        "starbridge.confluence.service.Service",
        "starbridge.hello.service.Service",
    ]

    for service in expected_services:
        assert service in result.stdout


def test_mcp_tools(runner):
    """Check available tools."""
    result = runner.invoke(cli, ["mcp", "tools"])
    assert result.exit_code == 0

    # All expected tool names
    expected_tools = [
        "starbridge_claude_health",
        "starbridge_claude_info",
        "starbridge_claude_restart",
        "starbridge_confluence_health",
        "starbridge_confluence_info",
        "starbridge_confluence_page_create",
        "starbridge_confluence_page_delete",
        "starbridge_confluence_page_get",
        "starbridge_confluence_page_list",
        "starbridge_confluence_page_update",
        "starbridge_confluence_space_list",
        "starbridge_hello_bridge",
        "starbridge_hello_health",
        "starbridge_hello_hello",
        "starbridge_hello_info",
        "starbridge_hello_pdf",
    ]

    output = result.stdout
    for tool in expected_tools:
        assert f"name='{tool}'" in output



from pathlib import Path

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


def test_hello_bridge(runner):
    """Check we dump the image."""
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["hello", "bridge", "--dump"])
        assert result.exit_code == 0
        assert Path("starbridge.png").is_file()
        assert Path("starbridge.png").stat().st_size == 6235


def test_hello_pdf(runner):
    """Check we dump the image."""
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["hello", "pdf", "--dump"])
        assert result.exit_code == 0
        assert Path("starbridge.pdf").is_file()
        assert Path("starbridge.pdf").stat().st_size == 6840



import base64
import os
from pathlib import Path

import pytest
from mcp import ClientSession, StdioServerParameters
from mcp.client.stdio import get_default_environment, stdio_client
from mcp.types import ImageContent, PromptMessage, TextContent
from typer.testing import CliRunner


@pytest.fixture
def runner():
    return CliRunner()


def _server_parameters(mocks: list[str] | None = None) -> StdioServerParameters:
    """Create server parameters with coverage enabled"""
    env = dict(get_default_environment())
    # Add coverage config to subprocess
    env.update({
        "COVERAGE_PROCESS_START": "pyproject.toml",
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", ".coverage"),
        "PYTHONPATH": ".",
    })
    if (mocks is not None) and mocks:
        env.update({"MOCKS": ",".join(mocks)})

    return StdioServerParameters(
        command="uv",
        args=["run", "starbridge"],
        env=env,
    )


@pytest.mark.asyncio
async def test_mcp_server_list_tools():
    """Test listing of tools from the server"""

    # Expected tool names that should be present
    expected_tools = [
        "starbridge_claude_health",
        "starbridge_claude_info",
        "starbridge_claude_restart",
        "starbridge_confluence_health",
        "starbridge_confluence_info",
        "starbridge_confluence_page_create",
        "starbridge_confluence_page_delete",
        "starbridge_confluence_page_get",
        "starbridge_confluence_page_list",
        "starbridge_confluence_page_update",
        "starbridge_confluence_space_list",
        "starbridge_hello_bridge",
        "starbridge_hello_health",
        "starbridge_hello_hello",
        "starbridge_hello_info",
        "starbridge_hello_pdf",
    ]

    async with stdio_client(_server_parameters()) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available tools
            result = await session.list_tools()

            # Verify each expected tool is present
            tool_names = [tool.name for tool in result.tools]
            for expected_tool in expected_tools:
                assert expected_tool in tool_names


@pytest.mark.asyncio
async def test_mcp_server_list_resources():
    async with stdio_client(
        _server_parameters(["atlassian.Confluence.get_all_spaces"])
    ) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available resources
            result = await session.list_resources()

            assert result.resources is not None
            assert len(result.resources) == 1
            assert any(
                resource.name == "helmut"
                for resource in result.resources
                if str(resource.uri)
                == "starbridge://confluence/space/~7120201709026d2b41448e93bb58d5fa301026"
            )


@pytest.mark.asyncio
async def test_mcp_server_list_prompts():
    """Test listing of prompts from the server"""
    async with stdio_client(_server_parameters()) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available prompts
            result = await session.list_prompts()

            assert result.prompts is not None


@pytest.mark.asyncio
async def test_mcp_server_prompt_get():
    """Test getting prompt from server"""
    async with stdio_client(
        _server_parameters(["atlassian.Confluence.get_all_spaces"])
    ) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available prompts
            result = await session.get_prompt("starbridge_confluence_space_summary")
            assert len(result.messages) == 1
            message = result.messages[0]
            assert type(message) is PromptMessage
            assert type(message.content) is TextContent
            assert (
                message.content.text
                == "Here are the current spaces to summarize:\n\n- ~7120201709026d2b41448e93bb58d5fa301026: helmut (personal)"
            )


@pytest.mark.asyncio
async def test_mcp_server_tool_call():
    """Test listing of prompts from the server"""
    async with stdio_client(_server_parameters()) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available prompts
            result = await session.call_tool("starbridge_hello_hello", {})
            assert len(result.content) == 1
            content = result.content[0]
            assert type(content) is TextContent
            assert content.text == "Hello World!"


@pytest.mark.asyncio
async def test_mcp_server_tool_call_with_image():
    """Test listing of prompts from the server"""
    async with stdio_client(_server_parameters()) as (read, write):
        async with ClientSession(read, write) as session:
            # Initialize the connection
            await session.initialize()

            # List available prompts
            result = await session.call_tool("starbridge_hello_bridge", {})
            assert len(result.content) == 1
            content = result.content[0]
            assert type(content) is ImageContent
            # Check base64 encoded data returned equals what's found in starbridge.png, again base64 encoded
            assert content.data == base64.b64encode(
                Path("tests/fixtures/starbridge.png").read_bytes()
            ).decode("utf-8")



from enum import StrEnum
from typing import ClassVar, Self

from pydantic import BaseModel, computed_field, model_validator


class _HealthStatus(StrEnum):
    UP = "UP"
    DOWN = "DOWN"


class Health(BaseModel):
    Status: ClassVar[type[_HealthStatus]] = _HealthStatus
    status: _HealthStatus
    reason: str | None = None

    @model_validator(mode="after")
    def up_has_no_reason(self) -> Self:
        if (self.status == _HealthStatus.UP) and self.reason:
            raise ValueError(f"Health {self.status} must not have reason")
        return self

    def __str__(self):
        if self.status == _HealthStatus.DOWN and self.reason:
            return f"{self.status.value}: {self.reason}"
        return self.status.value


class AggregatedHealth(BaseModel):
    dependencies: dict[str, Health]

    @computed_field
    @property
    def healthy(self) -> bool:
        """Computed from dependencies' status"""
        return all(
            health.status == Health.Status.UP for health in self.dependencies.values()
        )

    def __str__(self):
        status = "UP" if self.healthy else "DOWN"
        details = [
            f"{name}: {str(health)}" for name, health in self.dependencies.items()
        ]
        return f"{status} ({', '.join(details)})"
